<?php
/**
 * Created by PhpStorm.
 * User: TREULIAUX
 * Date: 12/12/2016
 * Time: 18:10
 */

/**
 * @file
 * Contains \Drupal\media_upload\Form\UploadMediaForm.
 */

namespace Drupal\media_upload\Form;

use Drupal\Core\Form\FormBase;
use Drupal\media_entity\Entity\Media;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Component\Utility\Bytes;

/**
 * Contribute form.
 */
class UploadMediaForm extends FormBase {

  const COMPLETE_FILE_NAME = 0;

  const FILE_NAME = 1;

  const EXT_NAME = 2;

  const FILENAME_REGEX = '/(^[\w\-\. ]+)\.([a-zA-Z0-9]+)/';

  private $_allowedImgExt = [];

  private $_allowedDocExt = [];

  private $_allowedVidExt = [];

  private $_allowedAudExt = [];

  private $_imgMaxSize = [];

  private $_docMaxSize = [];

  private $_vidMaxSize = [];

  private $_audMaxSize = [];

  private $_totalMaxSize = 0;

  private $_singleFileMaxSize = 0;

  private $_mediaBundle = [];

  public function __construct() {
    $config = $this->config('media_upload.settings');

    $this->_allowedImgExt = explode(' ', $config->get('image_ext'));
    $this->_allowedDocExt = explode(' ', $config->get('document_ext'));
    $this->_allowedVidExt = explode(' ', $config->get('video_ext'));
    $this->_allowedAudExt = explode(' ', $config->get('audio_ext'));

    $this->_imgMaxSize = Bytes::toInt($config->get('image_size'));
    $this->_docMaxSize = Bytes::toInt($config->get('document_size'));
    $this->_vidMaxSize = Bytes::toInt($config->get('video_size'));
    $this->_audMaxSize = Bytes::toInt($config->get('audio_size'));

    $this->_totalMaxSize = $config->get('total_size') * 1024 * 1024;
    // Note : DropZone take max_file_size as MB, and not B (hence no multiplication)
    $this->_singleFileMaxSize = $config->get('single_size') * 1;

    $this->_mediaBundle = [
      'image'    => [
        'format' => ($config->get('bundles')['image'] ? $this->_allowedImgExt : ''),
        'bundle' => $config->get('image_bundle'),
        'field'  => $config->get('image_field'),
      ],
      'video'    => [
        'format' => ($config->get('bundles')['video'] ? $this->_allowedVidExt : ''),
        'bundle' => $config->get('video_bundle'),
        'field'  => $config->get('video_field'),
      ],
      'document' => [
        'format' => ($config->get('bundles')['document'] ? $this->_allowedDocExt : ''),
        'bundle' => $config->get('document_bundle'),
        'field'  => $config->get('document_field'),
      ],
      'audio'    => [
        'format' => ($config->get('bundles')['audio'] ? $this->_allowedAudExt : ''),
        'bundle' => $config->get('audio_bundle'),
        'field'  => $config->get('audio_field'),
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {

    return 'mediaupload_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    if (empty(array_filter($this->config('media_upload.settings')
                                ->get('bundles')))
    ) {
      return drupal_set_message($this->t('No media bundle is enabled in module configuration'), 'warning');
    }

    $form['dropzonejs'] = [
      '#type'                 => 'dropzonejs',
      '#title'                => $this->t('Dropzone'),
      '#required'             => TRUE,
      '#dropzone_description' => $this->t('Click or drop your files here'),
      '#max_filesize'         => $this->_singleFileMaxSize === 0 ? '' : strval($this->_singleFileMaxSize) . 'M',
      '#extensions'           => trim(implode(' ', $this->_allowedDocExt) . ' ' . implode(' ', $this->_allowedImgExt) . ' ' . implode(' ', $this->_allowedVidExt) . ' ' . implode(' ', $this->_allowedAudExt)),
    ];
    $form['submit'] = [
      '#type'  => 'submit',
      '#value' => t('Submit'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    try {
      $errorFlag = FALSE;
      $fileCount = 0;
      $values = $form_state->getValues();
      if (!empty($values['dropzonejs']) && !empty($values['dropzonejs']['uploaded_files'])) {
        $files = $values['dropzonejs']['uploaded_files'];

        if (!$this->checkTotalSize($files)) {
          \Drupal::logger('mediaupload')
                 ->error($this->t('The total size of uploaded files is exceeded') . '(' . ($this->_totalMaxSize / 1024 / 1024) . 'MB)');

          return drupal_set_message($this->t('The total size of uploaded files is exceeded') . ' (' . ($this->_totalMaxSize / 1024 / 1024) . 'MB)', 'error');
        }

        foreach ($files as $file) {
          $fileInfo = [];
          if (preg_match(self::FILENAME_REGEX, $file['filename'], $fileInfo) === 1) {
            if (($bundle = $this->getBundleForFile($fileInfo[self::EXT_NAME])) !== FALSE) {
              if ($this->checkFileSize($file['path'], $bundle)) {
                $data = file_get_contents($file['path']);
                if (file_exists('public://' . $file['filename'])) {
                  \Drupal::logger('mediaupload')
                         ->notice($this->t('File @filename already exists - It has been replaced', ['@filename' => $file['filename']]));
                  drupal_set_message($this->t('File @filename already exists - It has been replaced', ['@filename' => $file['filename']]));
                }
                $file_entity = file_save_data($data, 'public://' . $file['filename'], FILE_EXISTS_REPLACE);
                $media = Media::create($this->getFieldsForMedia($fileInfo, $bundle, $file_entity));
                $media->save();
                $fileCount++;
              }
              else {
                $errorFlag = TRUE;
                \Drupal::logger('mediaupload')
                       ->warning($this->t('The maximum size of the @filename file is exceeded', ['@filename' => $file['filename']]));
                drupal_set_message($this->t('The maximum size of the @filename file is exceeded', ['@filename' => $file['filename']]), 'warning');
              }
            }
            else {
              $errorFlag = TRUE;
              \Drupal::logger('mediaupload')
                     ->error($this->t('@filename - File extension is not allowed', ['@filename' => $file['filename']]));
              drupal_set_message($this->t('@filename - File extension is not allowed', ['@filename' => $file['filename']]), 'error');
            }
          }
          else {
            $errorFlag = TRUE;
            \Drupal::logger('mediaupload')
                   ->warning($this->t('@filename - Incorrect file name', ['@filename' => $file['filename']]));
            drupal_set_message($this->t('@filename - Incorrect file name', ['@filename' => $file['filename']]), 'warning');
          }
        }
        if ($errorFlag && !$fileCount) {
          \Drupal::logger('mediaupload')
                 ->warning($this->t('No documents were uploaded'));

          return drupal_set_message($this->t('No documents were uploaded'), 'warning');
        }
        else {
          if ($errorFlag) {
            \Drupal::logger('mediaupload')
                   ->info($this->t('Some documents have not been uploaded'));
            drupal_set_message($this->t('Some documents have not been uploaded'), 'warning');
            \Drupal::logger('mediaupload')
                   ->info($this->t('@fileCount documents have been uploaded', ['@fileCount' => $fileCount]));

            return drupal_set_message($this->t('@fileCount documents have been uploaded', ['@fileCount' => $fileCount]));
          }
          else {
            \Drupal::logger('mediaupload')
                   ->info($this->t('@fileCount documents have been uploaded', ['@fileCount' => $fileCount]));

            return drupal_set_message($this->t('@fileCount documents have been uploaded', ['@fileCount' => $fileCount]));
          }
        }
      }
      \Drupal::logger('mediaupload')
             ->warning($this->t('No documents were uploaded'));

      return drupal_set_message($this->t('No documents were uploaded'), 'warning');
    } catch (\Exception $e) {
      \Drupal::logger('mediaupload')
             ->critical($e->getMessage());

      return drupal_set_message($e->getMessage(), 'error');
    }
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {

    return ['media_upload.settings'];
  }

  /**
   * @param array $files Files list to import
   *
   * @return bool
   */
  private function checkTotalSize(array $files) {
    if ($this->_totalMaxSize === 0) {
      return TRUE;
    }
    $totalSize = 0;
    foreach ($files as $file) {
      $totalSize += stat($file['path'])[7];
    }
    if ($totalSize > $this->_totalMaxSize) {
      return FALSE;
    }

    return TRUE;
  }

  /**
   * Check the size of a file
   *
   * @param $filePath     string File path
   * @param $bundleType   string Bundle type (index of $this->>mediaBundle)
   *
   * @return bool
   */
  private function checkFileSize($filePath, $bundleType) {
    $size = stat($filePath)[7];
    switch ($bundleType) {
      case 'image' :
        return ($this->_imgMaxSize == 0 || $size <= $this->_imgMaxSize) ? TRUE : FALSE;
        break;
      case 'video' :
        return ($this->_vidMaxSize == 0 || $size <= $this->_vidMaxSize) ? TRUE : FALSE;
        break;
      case 'document' :
        return ($this->_docMaxSize == 0 || $size <= $this->_docMaxSize) ? TRUE : FALSE;
        break;
      case 'audio' :
        return ($this->_audMaxSize == 0 || $size <= $this->_audMaxSize) ? TRUE : FALSE;
        break;
      default :
        return FALSE;
    }
  }

  /**
   * @param $fileExt string File extension
   *
   * @return string|FALSE
   */
  private function getBundleForFile($fileExt) {
    $bundle = FALSE;
    foreach ($this->_mediaBundle as $bundleType => $acceptedFormats) {
      if (in_array($fileExt, $acceptedFormats['format'])) {
        $bundle = $bundleType;
        break;
      }
    }

    return $bundle;
  }

  /**
   * @param $fileInfo        array
   * @param $bundleType      string Bundle type (index of $this->>mediaBundle)
   * @param $file_entity     \Drupal\file\FileInterface
   *
   * @return array
   */
  private function getFieldsForMedia(
    array $fileInfo,
    $bundleType,
    $file_entity
  ): array {
    $fieldFile = $this->_mediaBundle[$bundleType]['field'];
    $fields = [
      'bundle'   => $this->_mediaBundle[$bundleType]['bundle'],
      'name'     => $fileInfo[self::FILE_NAME],
      $fieldFile => [
        'target_id' => $file_entity->id(),
        'title'     => $fileInfo[self::FILE_NAME],
      ],
    ];

    return $fields;
  }
}
